import  click
import  pudb
from    pfmongo         import  driver, env
from    argparse        import  Namespace
from    pfmongo.models  import  responseModel
from    pfmisc          import  Colors as C
import  copy

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN
PL  = C.PURPLE
YL  = C.YELLOW

from pfmongo.models.dataModel import messageType

def options_add(id:str, options:Namespace) -> Namespace:
    localoptions:Namespace  = copy.deepcopy(options)
    localoptions.do         = 'getDocument'
    localoptions.argument   = id
    return localoptions

def get_envFailCheck(options:Namespace) -> int:
    if env.env_failCheck(options):
        return 100
    return 0

def documentGet_asInt(options) -> int:
    fail:int    = 0
    if(fail := get_envFailCheck(options)):
        return fail
    return driver.run_intReturn(options)

def documentGet_asModel(options) -> responseModel.mongodbResponse:
    model:responseModel.mongodbResponse = responseModel.mongodbResponse()
    if get_envFailCheck(options):
        model.message   = 'env failure'
        return model
    return driver.run_modelReturn(options)

@click.command(cls = env.CustomCommand, help=f"""
read and print {PL}document{NC} from a collection

SYNOPSIS
{CY}get {YL}--id <id>{NC}

DESC
This subcommand gets a document with passed 'id' from a collection and
prints it to the console. Use shell redirection to save to disk.

The "location" is defined by the core parameters, 'useDB' and 'useCollection'
which are typically defined in the CLI, in the system environment, or in the
session state.

""")
@click.option('--id',
    type  = str,
    help  = \
    "the document 'id' to get",
    default = '')
@click.pass_context
def get(ctx:click.Context, id:str="") -> int:
    # pudb.set_trace()
    return documentGet_asInt(options_add(id, ctx.obj['options']))



import click
import pudb
from pfmongo import driver, env
import pfmongo.pfmongo
from argparse import Namespace
import json
from pfmisc import Colors as C, switch
from pfmongo.config import settings
from pfmongo.models import responseModel
from typing import Tuple, cast, Callable
from pfmongo.commands.clop import connect
import copy

NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN
PL = C.PURPLE
YL = C.YELLOW

from pfmongo.models.dataModel import messageType


def options_add(file: str, id: str, options: Namespace) -> Namespace:
    localoptions: Namespace = copy.deepcopy(options)
    localoptions.do = "addDocument"
    localoptions.argument = {"file": file, "id": id}
    return localoptions


def flatten_dict(data: dict, parent_key: str = "", sep: str = "/") -> dict:
    flattened: dict = {}
    for k, v in data.items():
        new_key = f"{parent_key}{sep}{k}" if parent_key else k
        if isinstance(v, dict):
            flattened.update(flatten_dict(v, new_key, sep=sep))
        elif isinstance(v, list):
            for i, item in enumerate(v):
                list_key = f"{new_key}{sep}{i}"
                if isinstance(item, (dict, list)):
                    flattened.update(
                        flatten_dict({str(i): item}, parent_key=list_key, sep=sep)
                    )
                else:
                    flattened[list_key] = item
        else:
            flattened[new_key] = v
    return flattened


def env_OK(options: Namespace, d_doc: dict) -> bool | dict:
    envFailure: int = env.env_failCheck(options)
    if envFailure:
        return False
    if not d_doc["status"]:
        return not bool(env.complain(d_doc["data"], 1, messageType.ERROR))
    if "data" in d_doc:
        return d_doc["data"]
    else:
        return False


def jsonFile_intoDictRead(filename: str) -> dict[bool, dict]:
    d_json: dict = {"status": False, "filename": filename, "data": {}}
    try:
        f = open(filename)
        d_json["data"] = json.load(f)
        d_json["status"] = True
    except Exception as e:
        d_json["data"] = str(e)
    return d_json


def prepCollection_forDocument(
    options: Namespace,
    connectCollection: Callable[[Namespace], Namespace],
    document: dict,
) -> Callable[..., int | responseModel.mongodbResponse]:
    document["_date"] = pfmongo.pfmongo.timenow()
    document["_owner"] = settings.mongosettings.MD_sessionUser
    document["_size"] = driver.get_size(document)
    return driver.event_setup(
        driver.settmp(
            options,
            [
                {"collectionName": connectCollection(options).collectionName},
                {"argument": document},
            ],
        )
    )


def add_asType(
    document: dict, options: Namespace, modelReturnType: str
) -> int | responseModel.mongodbResponse:
    # First save to the shadow collection:
    shadowResp: responseModel.mongodbResponse = responseModel.mongodbResponse()
    if not settings.appsettings.donotFlatten:
        run = prepCollection_forDocument(
            options, connect.shadowCollection_getAndConnect, flatten_dict(document)
        )
        saveShadowFail: int | responseModel.mongodbResponse = run(
            printResponse=False, returnType="model"
        )
        shadowResp = cast(responseModel.mongodbResponse, saveShadowFail)
        if not shadowResp.status:
            match modelReturnType:
                case "model":
                    return saveShadowFail
                case "int":
                    return int(not shadowResp.status)
    if not options.argument["id"] and shadowResp.status:
        document["_id"] = shadowResp.response["connect"].documentName
    # Now save to the primary collection
    run = prepCollection_forDocument(
        options, connect.baseCollection_getAndConnect, document
    )
    return run(printResponse=True, returnType=modelReturnType)


def setup(options: Namespace) -> Tuple[int, dict]:
    d_data: dict = {}
    if env.env_failCheck(options):
        return 100, d_data
    d_dataOK: dict | bool = env_OK(
        options, jsonFile_intoDictRead(options.argument["file"])
    )
    if not d_dataOK:
        return 100, d_data
    if not isinstance(d_dataOK, dict):
        return 101, d_data
    d_data = d_dataOK
    if len(options.argument["id"]):
        d_data["_id"] = options.argument["id"]
    return 0, d_data


def earlyFailure(
    failData: Tuple[int, dict], returnType: str = "int"
) -> int | responseModel.mongodbResponse:
    reti: int = failData[0]
    retm: responseModel.mongodbResponse = responseModel.mongodbResponse()
    retm.message = f"A setup failure of return {reti} occurred"
    match returnType:
        case "int":
            return reti
        case "model":
            return retm
        case _:
            return reti


def documentAdd_asType(
    options: Namespace, returnType: str = "int"
) -> int | responseModel.mongodbResponse:
    failOrOK: Tuple[int, dict] = (-1, {})
    if (failOrOK := setup(options))[0]:
        return earlyFailure(failOrOK, returnType)
    d_data: dict = failOrOK[1]
    return add_asType(d_data, options, returnType)


def documentAdd_asInt(options: Namespace) -> int:
    return cast(int, documentAdd_asType(options, "int"))


def documentAdd_asModel(options: Namespace) -> responseModel.mongodbResponse:
    return cast(responseModel.mongodbResponse, documentAdd_asType(options, "model"))


@click.command(
    cls=env.CustomCommand,
    help=f"""
read a {PL}document{NC} from the filesystem and add to a collection

SYNOPSIS
{CY}add {YL}--file <filename> [--id <value>]{NC}

DESC
This subcommand accepts a document {YL}filename{NC} (assumed to contain JSON
formatted contents) and stores the contents in the associated {YL}COLLECTION{NC}.

A "shadow" document with a flat dataspace is also added to a "shadow"
collection. This "shadow" document facilitates searching and is kept
"in sync" with the orginal.

The "location" is defined by the core parameters, 'useDB' and 'useCollection'
which are typically defined in the CLI, in the system environment, or in the
session state.

""",
)
@click.option(
    "--file",
    type=str,
    help="The name of a JSON formatted file to save to a COLLECTION in a DATABASE.",
)
@click.option(
    "--id",
    type=str,
    help="If specified, set the 'id' in the mongo collection to the passed string.",
    default="",
)
@click.pass_context
def add(ctx: click.Context, file: str, id: str = "") -> int:
    # pudb.set_trace()
    return documentAdd_asInt(options_add(file, id, ctx.obj["options"]))



import  click
from    pfmongo         import  driver, env
from    argparse        import  Namespace
from    pfmisc          import  Colors as C
from    pfmongo.models  import  responseModel
import  copy
import  pudb

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN
PL  = C.PURPLE
YL  = C.YELLOW

def options_add(field:str, options:Namespace) -> Namespace:
    localoptions:Namespace  = copy.deepcopy(options)
    localoptions.do         = 'listDocument'
    localoptions.argument   = field
    return localoptions

def showAll_asInt(options:Namespace) -> int:
    return driver.run_intReturn(options)

def showAll_asModel(options:Namespace) -> responseModel.mongodbResponse:
    return driver.run_modelReturn(options)

@click.command(cls = env.CustomCommand, help=f"""
list {PL}documents{NC} in a collection.

SYNPOSIS
{CY}showall {YL}[--field <field>]{NC}

DESC
List all the documents in a collection, showing the contents of the
document's {YL}<field>{NC} key (by default this is '{YL}_id{NC}').

""")
@click.option('--field',
    type        = str,
    help        = \
    "List the value of the named field",
    default     = '_id')
@click.pass_context
def showAll(ctx:click.Context, field:str) -> int:
    # pudb.set_trace()
    return showAll_asInt(options_add(field, ctx.obj['options']))



import  click
from    argparse        import  Namespace
from    pfmisc          import  Colors as C
from    pfmongo         import  driver, env
from    pfmongo.models  import responseModel
import  copy

NC = C.NO_COLOUR
GR = C.LIGHT_GREEN
CY = C.CYAN
YL = C.YELLOW
PL = C.PURPLE

def options_add(options:Namespace) -> Namespace:
    localoptions:Namespace  = copy.deepcopy(options)
    localoptions.do         = 'showAllCollections'
    return localoptions

def showAll_asInt(options:Namespace) -> int:
    return driver.run_intReturn(options)

def showAll_asModel(options:Namespace) -> responseModel.mongodbResponse:
    return driver.run_modelReturn(options)

@click.command(cls=env.CustomCommand, help=f"""
list {PL}COLLECTIONS{NC} containing data in a {PL}DATABASE{NC}.

SYNOPSIS
{CY}showall{NC}

DESC
This command shows all the collections available in a given database
in a mongodb server. It accepts no arguments.

""")
@click.pass_context
def showAll(ctx:click.Context) -> int:
    # pudb.set_trace()
    return showAll_asInt(options_add(ctx.obj['options']))



from    argparse        import Namespace
import  click
from    pfmisc          import Colors as C
from    pfmongo         import driver, env
from    pfmongo.models  import responseModel
from    copy            import deepcopy
import  pudb

NC = C.NO_COLOUR
GR = C.LIGHT_GREEN
CY = C.CYAN
YL = C.YELLOW
PL = C.PURPLE

def options_add(database:str, options:Namespace) -> Namespace:
    localoptions:Namespace   = deepcopy(options)
    localoptions.do          = 'connectDB'
    localoptions.argument    = database
    return localoptions

def connectTo_asInt(options:Namespace) -> int:
    return driver.run_intReturn(options)

def connectTo_asModel(options:Namespace) -> responseModel.mongodbResponse:
    return driver.run_modelReturn(options)

@click.command(cls = env.CustomCommand, help=f"""
associate a context with {PL}DATABASE{NC}

SYNOPSIS
{CY}connect {YL}<DATABASE>{NC}

DESC
This command connects to a mongo database called {YL}DATABASE{NC}.
A mongodb "server" can contain several "databases". A {YL}DATABASE{NC}
is the lowest (or first) level of organization in monogodb.

In order to do any operations on data, you first MUST connect to
a {PL}DATABASE{NC}.
""")
@click.argument('database',
                required = True)
@click.pass_context
def connect(ctx:click.Context, database:str) -> int:
    return connectTo_asInt(options_add(database, ctx.obj['options']))



import click
import pudb
from pfmongo import driver, env
from argparse import Namespace
from pfmongo.models import responseModel
from pfmisc import Colors as C
import copy

from pfmongo.commands.clop import connect

NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN
PL = C.PURPLE
YL = C.YELLOW


def options_add(target: str, options: Namespace) -> Namespace:
    localoptions: Namespace = copy.deepcopy(options)

    localoptions.do = "searchDocument"
    localoptions.argument = {
        "field": "_id",
        "searchFor": target.split(","),
        "collection": connect.baseCollection_getAndConnect(options).collectionName,
    }
    return localoptions


def sg_asInt(options: Namespace) -> int:
    return driver.run_intReturn(options)


def sg_asModel(options: Namespace) -> responseModel.mongodbResponse:
    return driver.run_modelReturn(options)


@click.command(
    cls=env.CustomCommand,
    help=f"""
sipgrep {YL}pattern{NC}

SYNOPSIS
{CY}sg {YL}<pattern>{NC}

DESC
A pale shadow of "ripgrep", "sipgrep" aims to "sip", or perform an extremely
primitive and simple search across documents, returning the names of documents
(or files) that contain the search pattern.
""",
)
@click.pass_context
@click.argument("pattern", required=True)
def sg(ctx: click.Context, pattern: str) -> int:
    # pudb.set_trace()
    return sg_asInt(options_add(pattern, ctx.obj["options"]))



from re import A
import sys
import socket
from argparse import Namespace, ArgumentParser, RawTextHelpFormatter
from pfmongo import __main__
from pfmongo.pfmongo import options_initialize
from pfmongo.commands import smash
import pudb
from typing import Any


TERMINATION_SEQUENCE = b"\r\n\r\n"


def parser_setup(str_desc: str = "") -> ArgumentParser:
    description: str = ""
    if len(str_desc):
        description = str_desc
    parser = ArgumentParser(
        description=description, formatter_class=RawTextHelpFormatter
    )

    parser.add_argument(
        "--host",
        type=str,
        default="",
        help="host name or IP of server",
    )

    parser.add_argument(
        "--port",
        type=str,
        default="",
        help="port on which remote server is listening",
    )

    parser.add_argument(
        "--server",
        default=False,
        action="store_true",
        help="If specified, run in server mode",
    )

    parser.add_argument(
        "--response",
        type=str,
        default="string",
        help="response type: either a 'string' or 'dict'",
    )

    parser.add_argument(
        "--msg", type=str, default="", help="message to transmit in client mode"
    )

    return parser


def parser_interpret(parser: ArgumentParser, *args, **kwargs) -> Namespace:
    """
    Interpret the list space of *args, or sys.argv[1:] if
    *args is empty
    """
    options: Namespace = Namespace()
    asModule: bool = False
    for k, v in kwargs.items():
        if k == "asModule":
            asModule = v
    if asModule:
        # Here, this code is used a module to another app
        # and we don't want to "interpret" the host app's
        # CLI.
        options, unknown = parser.parse_known_args()
        return options
    if len(args):
        if len(args[0]):
            if isinstance(args[0][0], list):
                options = parser.parse_args(args[0][0])
            elif isinstance(args[0][0], dict):
                options = parser.parse_args(parser_JSONinterpret(args[0][0]))
        else:
            options = parser.parse_args(sys.argv[1:])
    return options


def parser_JSONinterpret(d_JSONargs) -> list:
    """
    Interpret a JSON dictionary in lieu of CLI.

    For each <key>:<value> in the d_JSONargs, append to
    list two strings ["--<key>", "<value>"] and then
    argparse.
    """
    l_args = []
    for k, v in d_JSONargs.items():
        if isinstance(v, bool):
            if v:
                l_args.append("--%s" % k)
            continue
        l_args.append("--%s" % k)
        l_args.append("%s" % v)
    return l_args


class IPCclient:
    def __init__(self, host: str, port: str):
        self.clientSocket: socket.socket = socket.socket(
            socket.AF_INET, socket.SOCK_STREAM
        )
        self.clientSocket.connect((host, int(port)))

    def message_sendAndReceive(self, msg: str) -> dict[str, str]:
        resp: dict[str, str] = {"response": ""}
        result: str = ""
        try:
            self.clientSocket.sendall(msg.encode())
            response: bytes = b""
            while True:
                chunk: bytes = self.clientSocket.recv(1024)
                if not chunk:
                    break
                response += chunk
            if response:
                result = response.decode()
            else:
                result = "No response received"
            resp["response"] = result
        finally:
            self.clientSocket.close()

        return resp


class IPCserver:
    def __init__(self, host: str, port: str):
        self.serverSocket: socket.socket = socket.socket(
            socket.AF_INET, socket.SOCK_STREAM
        )
        self.serverSocket.bind((host, int(port)))
        self.serverSocket.listen(1)
        self.connection: socket.socket
        self.clientAddress: tuple[str, str]
        print(f"smashes server setup and listening on '{host}:{port}'")

    def response_process(self, incoming: str) -> str:
        response: str | bytes
        ret: str = ""
        mdbOptions: Namespace = options_initialize()
        response = smash.smash_execute(
            smash.command_parse(smash.command_get(mdbOptions, noninteractive=incoming))
        )
        if isinstance(response, str):
            ret = response
        if isinstance(response, bytes):
            ret = response.decode()
        print(f"{ret}")
        return ret

    def response_await(self) -> str:
        incoming: str = ""
        resp: str = ""
        self.connection, self.clientAddress = self.serverSocket.accept()
        try:
            print(f"Connection from {self.clientAddress}")
            data: bytes = b""
            data = self.connection.recv(32768)
            if data:
                incoming = data.decode()
                print(f"Received: {incoming}")
                self.connection.sendall(self.response_process(incoming).encode())
        finally:
            self.connection.close()

        return resp

    def start(self) -> None:
        while True:
            incoming: str = self.response_await()


def server_handle(options: Namespace) -> None:
    server: IPCserver = IPCserver(options.host, options.port)
    server.start()


def client_handle(options: Namespace) -> dict[str, str]:
    client: IPCclient = IPCclient(options.host, options.port)
    return client.message_sendAndReceive(options.msg)


def response_toConsole(resp: dict[str, str]) -> str:
    return resp["response"]


def main(*args: list[Any]) -> str | dict[str, str]:
    options: Namespace = parser_interpret(parser_setup(), args)

    dresp: dict[str, str]
    if options.server:
        server_handle(options)

    dresp = client_handle(options)
    if "string" in options.response:
        return response_toConsole(dresp)
    return dresp


if __name__ == "__main__":
    main()



import  click
from    pfmongo         import  driver, env
from    argparse        import  Namespace
from    pfmisc          import  Colors as C
from    pfmongo.models  import  responseModel, fsModel
from    pathlib         import  Path
import  copy
import  ast
import  pudb

from    pfmongo.commands.document   import showAll as doc
from    pfmongo.commands.state      import showAll as state
import  pfmongo.commands.smash      as smash
import  pfmongo.commands.fop.cd     as cd


NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN
PL  = C.PURPLE
YL  = C.YELLOW

def options_add(options:Namespace) -> Namespace:
    localoptions:Namespace              = copy.deepcopy(options)
    localoptions.beQuiet                = True
    cwd:responseModel.mongodbResponse   = state.showAll_asModel(localoptions)
    localoptions.cwd                    = '/' +  cwd.message
    return localoptions

def cd_options(options:Namespace) -> Namespace:
    mkdir:bool  = True
    return cd.options_add(options.cwd, options, mkdir)

def prompt_do(options:Namespace) -> fsModel.cdResponse:
    promptResp:fsModel.cdResponse       = cd.changeDirectory(cd_options(options))
    (db, collection)                    = cd.path_to_dbCol(cd_options(options))
    if db:
        promptResp.message  = '/'
        match promptResp.state['database']:
            case 'void':    promptResp.message  += f"{YL}{db}{NC}"
            case 'exists':  promptResp.message  += f"{GR}{db}{NC}"
    if collection:
        promptResp.message += '/'
        match promptResp.state['collection']:
            case 'void':    promptResp.message  += f"{YL}{collection}{NC}"
            case 'exists':  promptResp.message  += f"{GR}{collection}{NC}"
    return promptResp

def prompt_asModel(options) -> fsModel.cdResponse:
    return prompt_do(options)

def prompt_asInt(options) -> int:
    pathResp:fsModel.cdResponse = prompt_do(options)
    print(pathResp.message)
    return pathResp.code

@click.command(cls = env.CustomCommand, help=f"""
return a {YL}color {CY}aware{NC} prompt

SYNOPSIS
{CY}prompt{NC}

DESC
This command returns a prompt where the color of a directory indicates its
existence in the monogo server: {YL}yellow{NC} indicates the directory is connected,
but empty; {GR}green{NC} indicates connected and has contents.

Only mongodb "directories" that have contents are listable. What this means
in terms of file system analogues is that in a file system a {YL}mkdir{NC}
command actually creates something that at first is empty. In monogo, a
{YL}database{NC} or {YL}collection{NC} is only "really real" once it is not empty.

""")
@click.pass_context
def prompt(ctx:click.Context) -> int:
    # pudb.set_trace()
    return prompt_asInt(options_add(ctx.obj['options']))
